@@ -106,7 +106,7 @@ def _fetch_response(method, extra_params):
 
 def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_url=None,
             action=SALE, user=None, user_address=None, shipping_method=None,
-            shipping_address=None, no_shipping=False):
+            shipping_address=None, no_shipping=False, paypal_params=None):
     """
     Register the transaction with PayPal to get a token which we use in the
     redirect URL.  This is the 'SetExpressCheckout' from their documentation.
@@ -114,25 +114,72 @@ def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_u
     There are quite a few options that can be passed to PayPal to configure
     this request - most are controlled by PAYPAL_* settings.
     """
+    # Default parameters (taken from global settings).  These can be overridden
+    # and customised using the paypal_params parameter.
+    _params = {
+        'CUSTOMERSERVICENUMBER': getattr(
+            settings, 'PAYPAL_CUSTOMER_SERVICES_NUMBER', None),
+        'SOLUTIONTYPE': getattr(settings, 'PAYPAL_SOLUTION_TYPE', None),
+        'LANDINGPAGE': getattr(settings, 'PAYPAL_LANDING_PAGE', None),
+        'BRANDNAME': getattr(settings, 'PAYPAL_BRAND_NAME', None),
+
+        # Display settings
+        'PAGESTYLE': getattr(settings, 'PAYPAL_PAGESTYLE', None),
+        'HDRIMG': getattr(settings, 'PAYPAL_HEADER_IMG', None),
+        'PAYFLOWCOLOR': getattr(settings, 'PAYPAL_PAYFLOW_COLOR', None),
+
+        # Think these settings maybe deprecated in latest version of PayPal's
+        # API
+        'HDRBACKCOLOR': getattr(settings, 'PAYPAL_HEADER_BACK_COLOR', None),
+        'HDRBORDERCOLOR': getattr(
+            settings, 'PAYPAL_HEADER_BORDER_COLOR', None),
+
+        'LOCALECODE': getattr(settings, 'PAYPAL_LOCALE', None),
+
+        'ALLOWNOTE': getattr(settings, 'PAYPAL_ALLOW_NOTE', True),
+        'CALLBACKTIMEOUT': getattr(settings, 'PAYPAL_CALLBACK_TIMEOUT', 3)
+    }
+    confirm_shipping_addr = getattr(settings, 'PAYPAL_CONFIRM_SHIPPING', None)
+    if confirm_shipping_addr and not no_shipping:
+        _params['REQCONFIRMSHIPPING'] = 1
+    if paypal_params:
+        _params.update(paypal_params)
+
+    locale = _params.get('LOCALECODE', None)
+    if locale:
+        valid_choices = ('AU', 'DE', 'FR', 'GB', 'IT', 'ES', 'JP', 'US')
+        if locale not in valid_choices:
+            raise ImproperlyConfigured(
+                "'%s' is not a valid locale code" % locale)
+
+    # Boolean values become integers
+    _params.update((k, int(v)) for k, v in _params.iteritems() if isinstance(v, bool))
+
+    # Remove None values
+    params = dict((k, v) for k, v in _params.iteritems() if v is not None)
+
     # PayPal have an upper limit on transactions.  It's in dollars which is a
     # fiddly to work with.  Lazy solution - only check when dollars are used as
     # the PayPal currency.
     amount = basket.total_incl_tax
     if currency == 'USD' and amount > 10000:
-        raise exceptions.PayPalError(
-            'PayPal can only be used for orders up to 10000 USD')
+        msg = 'PayPal can only be used for orders up to 10000 USD'
+        logger.error(msg)
+        raise exceptions.PayPalError(msg)
 
     if amount <= 0:
-        raise exceptions.PayPalError('Zero value basket is not allowed')
+        msg = 'Zero value basket is not allowed'
+        logger.error(msg)
+        raise exceptions.PayPalError(msg)
 
     # PAYMENTREQUEST_0_AMT should include tax, shipping and handling
-    params = {
+    params.update({
         'PAYMENTREQUEST_0_AMT': amount,
         'PAYMENTREQUEST_0_CURRENCYCODE': currency,
         'RETURNURL': return_url,
         'CANCELURL': cancel_url,
         'PAYMENTREQUEST_0_PAYMENTACTION': action,
-    }
+    })
 
     # Add item details
     index = 0
@@ -202,49 +249,9 @@ def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_u
         basket.total_incl_tax)
     params['PAYMENTREQUEST_0_TAXAMT'] = _format_currency(D('0.00'))
 
-    # Customer services number
-    customer_service_num = getattr(
-        settings, 'PAYPAL_CUSTOMER_SERVICES_NUMBER', None)
-    if customer_service_num:
-        params['CUSTOMERSERVICENUMBER'] = customer_service_num
-
-    # Display settings
-    page_style = getattr(settings, 'PAYPAL_PAGESTYLE', None)
-    header_image = getattr(settings, 'PAYPAL_HEADER_IMG', None)
-    if page_style:
-        params['PAGESTYLE'] = page_style
-    elif header_image:
-        params['LOGOIMG'] = header_image
-    else:
-        # Think these settings maybe deprecated in latest version of PayPal's
-        # API
-        display_params = {
-            'HDRBACKCOLOR': getattr(settings,
-                                    'PAYPAL_HEADER_BACK_COLOR', None),
-            'HDRBORDERCOLOR': getattr(settings,
-                                      'PAYPAL_HEADER_BORDER_COLOR', None),
-        }
-        params.update(x for x in display_params.items() if bool(x[1]))
-
-    # Locale
-    locale = getattr(settings, 'PAYPAL_LOCALE', None)
-    if locale:
-        valid_choices = ('AU', 'DE', 'FR', 'GB', 'IT', 'ES', 'JP', 'US')
-        if locale not in valid_choices:
-            raise ImproperlyConfigured(
-                "'%s' is not a valid locale code" % locale)
-        params['LOCALECODE'] = locale
-
-    # Confirmed shipping address
-    confirm_shipping_addr = getattr(settings, 'PAYPAL_CONFIRM_SHIPPING', None)
-    if confirm_shipping_addr:
-        params['REQCONFIRMSHIPPING'] = 1
-
     # Instant update callback information
     if update_url:
         params['CALLBACK'] = update_url
-        params['CALLBACKTIMEOUT'] = getattr(
-            settings, 'PAYPAL_CALLBACK_TIMEOUT', 3)
 
     # Contact details and address details - we provide these as it would make
     # the PayPal registration process smoother is the user doesn't already have
@@ -252,7 +259,7 @@ def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_u
     if user:
         params['EMAIL'] = user.email
     if user_address:
-        params['SHIPTONAME'] = user_address.name()
+        params['SHIPTONAME'] = user_address.name
         params['SHIPTOSTREET'] = user_address.line1
         params['SHIPTOSTREET2'] = user_address.line2
         params['SHIPTOCITY'] = user_address.line4
@@ -267,28 +274,24 @@ def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_u
         # It's recommend not to set 'confirmed shipping' if supplying the
         # shipping address directly.
         params['REQCONFIRMSHIPPING'] = 0
-        params['SHIPTONAME'] = shipping_address.name()
+        params['SHIPTONAME'] = shipping_address.name
         params['SHIPTOSTREET'] = shipping_address.line1
         params['SHIPTOSTREET2'] = shipping_address.line2
         params['SHIPTOCITY'] = shipping_address.line4
         params['SHIPTOSTATE'] = shipping_address.state
         params['SHIPTOZIP'] = shipping_address.postcode
         params['SHIPTOCOUNTRYCODE'] = shipping_address.country.iso_3166_1_a2
+
     elif no_shipping:
         params['NOSHIPPING'] = 1
 
-    # Allow customer to specify a shipping note
-    allow_note = getattr(settings, 'PAYPAL_ALLOW_NOTE', True)
-    if allow_note:
-        params['ALLOWNOTE'] = 1
-
     # Shipping charges
     params['PAYMENTREQUEST_0_SHIPPINGAMT'] = _format_currency(D('0.00'))
     max_charge = D('0.00')
     for index, method in enumerate(shipping_methods):
         is_default = index == 0
         params['L_SHIPPINGOPTIONISDEFAULT%d' % index] = 'true' if is_default else 'false'
-        charge = method.basket_charge_incl_tax()
+        charge = method.charge_incl_tax
         if charge > max_charge:
             max_charge = charge
         if is_default:
@@ -299,7 +302,7 @@ def set_txn(basket, shipping_methods, currency, return_url, cancel_url, update_u
 
     # Set shipping charge explicitly if it has been passed
     if shipping_method:
-        max_charge = charge = shipping_method.basket_charge_incl_tax()
+        max_charge = charge = shipping_method.charge_incl_tax
         params['PAYMENTREQUEST_0_SHIPPINGAMT'] = _format_currency(charge)
         params['PAYMENTREQUEST_0_AMT'] += charge
 
